# frozen_string_literal: true

class PrepareCsvFromCrcFolder
  attr_reader :parser, :csv_file_path

  def initialize(parser)
    @parser = parser
    @load_or_create_session_per_subject = {}
    @system_created_sessions = []
  end

  def prepare_csv!
    FileUtils.mkdir_p(csv_folder_path)

    csv_file = File.new(csv_file_path, 'w')

    CSV.open(csv_file, 'wb') do |csv|
      csv << csv_file_headers

      Dir.glob("#{parser.importer.parser_fields['import_file_path']}/*").each do |group_folder_path|
        prepare_group_data(group_folder_path, csv)
      end
    end
  end

  def csv_file_path
    @csv_file_path ||= "#{csv_folder_path}/folder_data.csv"
  end

  # Prepare the csv file to this path
  def csv_folder_path
    @csv_folder_path ||= File.join(Rails.root, parser.base_path,
                                   "#{parser.importer.id}_#{parser.importer.created_at.strftime('%Y%m%d%H%M%S')}")
  end

  def csv_file_headers
    %w[
      model
      collection_type
      crc_collection_type
      file
      source
      parents
      for_complex_identifier
      for_complex_type
      title
      contributor
      contributor_orcid
      contributor_affiliation
      contributor_scheme
      creator
      creator_orcid
      creator_affiliation
      creator_scheme
      keyword
      resource_type
      crc_resource_type
      folder_name
      experiment_description
      crc_work_type
      doi
      alternative_title
      description
      group_identifier
      record_date
      date_description
      date
      modality
      subject_identifier
      subject_species
      subject_type
      subject_sex
      subject_age
      approval_number
      subject
      publisher
      language
      funder_identifier
      funder_name
      award_number
      award_uri
      award_title
      coverage
      software_version
      extra_information
    ]
  end

  def crc_1280_collection_source_indentifier
    crc_collection_title = ENV.fetch('CRC_1280_COLLECTION', 'CRC 1280')
    collection_type = Hyrax::CollectionType.find_or_create_by(title: crc_collection_title)
    collection_type_gid = collection_type.gid

    crc_1280_collections = collection_type.collections.select { |t| t.title == [crc_collection_title] }

    return '' unless crc_1280_collections.any?

    crc_1280_collections.first.source.join
  end

  def prepare_group_data(group_folder_path, csv)
    crc_collection_title = ENV.fetch('CRC_1280_COLLECTION', 'CRC 1280')
    experiment_folders_path = Dir.glob("#{group_folder_path}/*").select { |item| Pathname(item).directory? }

    title = group_folder_path.split('/').last
    source_identifier = SecureRandom.uuid

    csv << ['Collection', crc_collection_title, 'group', '', source_identifier, crc_1280_collection_source_indentifier,'', '', title]

    return unless experiment_folders_path.present?

    experiment_folders_path.each do |experiment_folder_path|
      prepare_experiment_data(experiment_folder_path, csv, source_identifier)
    end
  end

  def prepare_experiment_data(experiment_folder_path, csv, parent_group_source_identifier)
    experiment_items = Dir.glob("#{experiment_folder_path}/*")

    json_file_path = find_meta_json_file_path_in_look_down(experiment_folder_path)

    return unless json_file_path.present?

    source_identifier = SecureRandom.uuid

    meta_info = {
      folder_name: experiment_folder_path.split('/').last,
      source_identifier: source_identifier,
      parent_group_source_identifier: parent_group_source_identifier,
      crc_work_type: 'experiment'
    }

    csv << map_work_meta_attributes_with_json_file(json_file_path, meta_info)

    create_file_sets_for_work(experiment_folder_path, csv, meta_info)

    return unless experiment_items.present?

    experiment_items.select { |item| Pathname(item).directory? }.each do |subject_folder_path|
      prepare_subject_data(subject_folder_path, csv, source_identifier)
    end
  end

  def prepare_subject_data(subject_folder_path, csv, parent_work_source_identifier)
    json_file_path = find_meta_json_file_path_in_look_down(subject_folder_path)

    return unless json_file_path.present?

    source_identifier = SecureRandom.uuid

    meta_info = {
      folder_name: subject_folder_path.split('/').last,
      source_identifier: source_identifier,
      parent_work_source_identifier: parent_work_source_identifier,
      crc_work_type: 'ComplexSubject'
    }

    csv << map_subject_attributes_with_json_file(json_file_path, meta_info)

    create_file_sets_for_work(subject_folder_path, csv, meta_info)

    find_or_create_session(subject_folder_path, csv, source_identifier, parent_work_source_identifier)
  end

  def prepare_session_data(session_folder_path, csv, parent_identifier, parent_work_source_identifier)
    session_items = Dir.glob("#{session_folder_path}/*")

    json_file_path = find_meta_json_file_path_in_look_down(session_folder_path)

    if json_file_path.nil?
      parent_path = File.dirname(session_folder_path)
      json_file_path = find_meta_json_file_path_in_look_up(parent_path, true)
    end

    return unless json_file_path.present?

    source_identifier = SecureRandom.uuid

    meta_info = {
      folder_name: session_folder_path.split('/').last,
      source_identifier: source_identifier,
      parent_source_identifier: parent_identifier,
      parent_work_source_identifier: parent_work_source_identifier,
      crc_work_type: 'ComplexSession'
    }

    csv << map_session_attributes_with_json_file(json_file_path, meta_info)

    create_file_sets_for_work(session_folder_path, csv, meta_info)

    return unless session_items.present?

    session_items.select { |item| Pathname(item).directory? }.each do |modality_folder_path|
      prepare_modality_data(modality_folder_path, csv, source_identifier, parent_work_source_identifier)
    end
  end

  def prepare_modality_data(modality_folder_path, csv, parent_identifier,  parent_work_source_identifier)
    json_file_path = find_meta_json_file_path_in_look_down(modality_folder_path)

    if json_file_path.nil?
      parent_path = File.dirname(modality_folder_path)
      json_file_path = find_meta_json_file_path_in_look_up(parent_path,
                                                           @system_created_sessions.include?(parent_work_source_identifier))
    end

    return unless json_file_path.present?

    source_identifier = SecureRandom.uuid

    meta_info = {
      folder_name: modality_folder_path.split('/').last,
      source_identifier: source_identifier,
      parent_source_identifier: parent_identifier,
      parent_work_source_identifier: parent_work_source_identifier,
      crc_work_type: 'ComplexModality'
    }

    csv << map_modality_attributes_with_json_file(json_file_path, meta_info)

    create_file_sets_for_work(modality_folder_path, csv, meta_info)
  end

  def find_or_create_session(subject_folder_path, csv, subject_source_identifier, parent_work_source_identifier)
    folders_path = Dir.glob("#{subject_folder_path}/*").select { |item| Pathname(item).directory? }

    return unless folders_path.present?

    folders_path.each do |folder_path|
      if folder_is_session_or_modality(folder_path) == 'session'
        prepare_session_data(folder_path, csv, subject_source_identifier, parent_work_source_identifier)
      else
        session = load_or_create_session_per_subject(folder_path, csv, subject_source_identifier, parent_work_source_identifier)

        next unless session

        session_source_identifier = session[4]

        prepare_modality_data(folder_path, csv, session_source_identifier, parent_work_source_identifier)
      end
    end
  end

  def load_or_create_session_per_subject(folder_path, csv, subject_source_identifier, parent_work_source_identifier)
    key = "#{subject_source_identifier}_session"

    unless @load_or_create_session_per_subject.key?(key)
      json_file_path = find_meta_json_file_path_in_look_down(folder_path)

      if json_file_path.nil?
        parent_path = File.dirname(folder_path)
        json_file_path = find_meta_json_file_path_in_look_up(parent_path, true)
      end

      return unless json_file_path.present?

      source_identifier = SecureRandom.uuid

      meta_info = {
        folder_name: 'SES 01',
        source_identifier: source_identifier,
        parent_source_identifier: subject_source_identifier,
        parent_work_source_identifier: parent_work_source_identifier,
        crc_work_type: 'ComplexSession'
      }

      @load_or_create_session_per_subject[key] = map_session_attributes_with_json_file(json_file_path, meta_info)
      @system_created_sessions << source_identifier

      csv << @load_or_create_session_per_subject[key]
    end

    @load_or_create_session_per_subject[key]
  end

  def find_meta_json_file_path_in_look_down(folder_path)
    meta_json_file_path = nil

    folder_items_path = Dir.glob("#{folder_path}/*")

    return nil unless folder_items_path.any?

    file_path = "#{folder_path}/meta.json"

    return file_path if folder_items_path.include?(file_path)

    folder_items_path.select { |item| Pathname(item).directory? }.each do |folder_item_path|
      meta_json_file_path = find_meta_json_file_path_in_look_down(folder_item_path)

      break if meta_json_file_path
    end

    meta_json_file_path
  end

  def find_meta_json_file_path_in_look_up(folder_path, is_lookup_subject_folder)
    folder_items_path = Dir.glob("#{folder_path}/*")

    return nil unless folder_items_path.any?

    file_path = "#{folder_path}/meta.json"

    return file_path if folder_items_path.include?(file_path)

    return nil if is_lookup_subject_folder

    parent_path = File.dirname(folder_path)

    find_meta_json_file_path_in_look_up(parent_path, true)
  end

  def map_work_meta_attributes_with_json_file(file_path, meta_info)
    file_attributes = JSON.parse(File.read(file_path).force_encoding('iso-8859-1').encode('utf-8'))

    # fix incorrect spelling in import data
    file_attributes['Modality'] = correct_spelling_of_modality(file_attributes['Modality'])

    [
      'CrcDataset',
      '',
      '',
      '',
      meta_info[:source_identifier],
      [meta_info[:parent_work_source_identifier], meta_info[:parent_group_source_identifier]].compact.join(';'),
      '',
      '',
      file_attributes['Experiment title'],
      file_attributes['Contributor'],
      file_attributes['DataCite-contributorNameIdentifier'],
      file_attributes['DataCite-contirbutorAffiliation'],
      file_attributes['DataCite-contributorNameIdentifierScheme'],
      file_attributes['Creator'],
      file_attributes['DataCite-creatorNameIdentifier'],
      file_attributes['DataCite-creatorAffiliation'],
      file_attributes['DataCite-creatorNameIdentifierScheme'],
      '',
      'Dataset',
      file_attributes['Resource Type'],
      meta_info[:folder_name],
      file_attributes['Experiment Description'],
      meta_info[:crc_work_type],
      '',
      '',
      file_attributes['DublinCore-Description'],
      file_attributes['Group ID'],
      file_attributes['Record date'],
      file_attributes['DataCite-dateType'],
      file_attributes['DataCite-Date'],
      file_attributes['Modality'],
      file_attributes['Subject ID'],
      file_attributes['Subject Species'],
      file_attributes['Subject type'],
      file_attributes['Subject sex'],
      file_attributes['Subject age'],
      file_attributes['Animal|Ethics approval No.'],
      file_attributes['DublinCore-Subject'],
      file_attributes['DublinCore-Publisher'],
      file_attributes['DublinCore-Language'],
      file_attributes['DataCite-funderIdentifier'],
      file_attributes['DataCite-FundingReference'],
      '',
      '',
      '',
      file_attributes['DublinCore-Coverage'],
      file_attributes['Software version'],
      file_attributes['Extra information']
    ]
  end

  def map_subject_attributes_with_json_file(file_path, meta_info)
    file_attributes = JSON.parse(File.read(file_path).force_encoding('iso-8859-1').encode('utf-8'))

    [
      'ComplexSubject',
      '',
      '',
      '',
      meta_info[:source_identifier],
      meta_info[:parent_work_source_identifier],
      '',
      '',
      meta_info[:folder_name],
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      file_attributes['Subject ID'],
      file_attributes['Subject Species'],
      file_attributes['Subject type'],
      file_attributes['Subject sex'],
      file_attributes['Subject age'],
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      ''
    ]
  end

  def map_session_attributes_with_json_file(file_path, meta_info)
    file_attributes = JSON.parse(File.read(file_path).force_encoding('iso-8859-1').encode('utf-8'))

    [
      'ComplexSession',
      '',
      '',
      '',
      meta_info[:source_identifier],
      meta_info[:parent_source_identifier],
      '',
      '',
      meta_info[:folder_name],
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      file_attributes['Record date'],
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      ''
    ]
  end

  def map_modality_attributes_with_json_file(file_path, meta_info)
    file_attributes = JSON.parse(File.read(file_path).force_encoding('iso-8859-1').encode('utf-8'))
    file_attributes['Modality'] = correct_spelling_of_modality(file_attributes['Modality'])
    [
      'ComplexModality',
      '',
      '',
      '',
      meta_info[:source_identifier],
      meta_info[:parent_source_identifier],
      '',
      '',
      meta_info[:folder_name],
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      file_attributes['Modality']&.join(','),
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      '',
      ''
    ]
  end

  def create_file_sets_for_work(folder_path, csv, meta_info)
    Dir.glob("#{folder_path}/*").reject { |item| Pathname(item).directory? || item.include?('MetaApp2_SFB1280.exe') || item.include?('meta.json') }.each_with_index do |file_path, index|
      for_complex_type = meta_info[:crc_work_type] == 'experiment' ? "CrcDataset" : meta_info[:crc_work_type]
      csv << ['FileSet', '', '', file_path, SecureRandom.uuid, meta_info[:parent_work_source_identifier] || meta_info[:source_identifier],  meta_info[:source_identifier], for_complex_type, file_path.split('/')[-1]]
    end
  end

  def folder_is_session_or_modality(folder_path)
    return 'session' if Dir.glob("#{folder_path}/*").select { |item| Pathname(item).directory? }.any?

    'modality'
  end

  def correct_spelling_of_modality(modality)
    modality.map do |value|
      case value
      when 'Behavorial'
        'Behavioral'
      when 'Questionaires'
        'Questionnaires'
      else
        value
      end
    end
  end
end
